function map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2)

% Compute a free-boundary conformal parameterization for point clouds with 
% disk topology using the method in [1].
%
% Usage:
% map = pc_free_boundary_conformal_map(v,bdy_index,k,c1,c2);
% 
% Input:
% v: nv x 3 vertex coordinates of a disk-shaped point cloud
% bdy_index: nb x 1 vertex indices of the boundary, with anti-clockwise orientation
% k: kNN parameter
% c1, c2: prescribed lower and upper bound for the boundary angles (in degrees)
%
% Output:
% map: nv x 2 vertex coordinates of the free boundary conformal parameterization
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, Y. Liu, and L. M. Lui,
%     "Free-boundary conformal parameterization of point clouds."
%     Journal of Scientific Computing, 90(1), 14, 2022.
%
% Copyright (c) 2020-2023, Gary P. T. Choi, Yechen Liu, Lok Ming Lui

% Find vertex ring
[rings, local_ring] = pc_find_vertex_ring(v,k,c1,c2);

% Obtain the point cloud Laplacian
nv = length(local_ring);
new_ring = cell(nv,1);
for i=1:nv
    if ~ismember(i,bdy_index)
        new_ring{i} = rings{i};
    else
        new_ring{i} = local_ring{i};
    end
end
row = cell(nv,1);
col = cell(nv,1);
vv = cell(nv,1); 
for ii = 1:nv
    ring = new_ring{ii};    
    tmp = 1/3 * cotmatrix(v,ring);    
    [row{ii},col{ii},vv{ii}] = find(tmp);
end
row = cell2mat(row);
col = cell2mat(col);
vv = cell2mat(vv);
L0 = sparse(row,col,vv);

% Construct the area matrix 
A = pc_area_matrix(bdy_index, nv);

% Solve linear system
[~, b] = fartwo(v);
c = [0, 0; 1, 0];
C = cell(2,1);
[C{:}] = deal(L0);
L = blkdiag(C{:});
map = reshape(laprow(L+2*A, [b(:); b(:)+nv], c(:)), nv, 2);

